//--------------------------------------------------------------------|
// Arduino acting as remote shutter switch for RICOH Cameras:         |
// RICOH GR Digital III                                               |
// RICOH GR Digital II                                                |
// RICOH GR Digital                                                   |	
// RICOH GX200                                                        | 	
// RICOH GXR                                                          | 	
// RICOH CX4                                                          | 	
// RICOH CX3                                                          | 	
// RICOH CX2                                                          | 	
// RICOH CX1                                                          | 	
// RICOH GX100                                                        | 	
// RICOH GX8                                                          |
// (Emulates RICOH CA-1 Cable switch)                                 |
//  - JUST A PROOF OF CONCEPT -                                       |
// http://infar.be                                                    |
// 2010                                                               |
// Share/use as you like, but please give a credit ;)                 |
//--------------------------------------------------------------------|

// Switches are connected from Arduino pins 3 and 4 to GND.
const int switchHalfPressPin = 3;    // Half press switch connected to Arduino pin 3
const int switchFullPressPin = 4;    // Full press switch connected to Arduino pin 4

// ...going from Arduino Pin 5 (via 1k series resistor) to RICOH camera USB+ line
const int cameraUsbPin = 5;

// Variables for button readout
int switchHalfPressState = 1;        // Initialize variables      
int switchFullPressState = 1;        // for button press

// State definitions and initialization
#define RELEASED ((unsigned char) 0)
#define HALFPRESSED ((unsigned char) 1)
#define FULLPRESSED ((unsigned char) 2)
unsigned char currentState = RELEASED;
unsigned char previousState = RELEASED;

void setup()
{
  pinMode(switchHalfPressPin, INPUT);        // Configure as input and   
  digitalWrite(switchHalfPressPin, HIGH);    // set to HIGH
  pinMode(switchFullPressPin, INPUT);        // Configure as input and
  digitalWrite(switchFullPressPin, HIGH);    // set to HIGH
  pinMode(cameraUsbPin, OUTPUT);             // Configure as output and
  digitalWrite(cameraUsbPin, LOW);           // set to LOW
}

void loop()
{
  switchHalfPressState = digitalRead(switchHalfPressPin);
  switchFullPressState = digitalRead(switchFullPressPin);

  if ((switchHalfPressState == 0) && (switchFullPressState == 1))
  {
    currentState = HALFPRESSED;
    Serial.println("HALFPRESSED");
  }
  if ((switchHalfPressState == 0) && (switchFullPressState == 0))
  {
    currentState = FULLPRESSED;
    Serial.println("FULLPRESSED");
  }
  if ((switchHalfPressState == 1) && (switchFullPressState == 1))
  {
    currentState = RELEASED;
    Serial.println("RELEASED");
  }
  
  if (previousState != currentState)  // When state changes
  {
      if (currentState == HALFPRESSED)
      {
        halfpress();
        previousState = HALFPRESSED;        
      }
      if (currentState == FULLPRESSED)
      {
        fullpress();
        previousState = FULLPRESSED;        
      }
      if (currentState == RELEASED)
      {
        release();
        previousState = RELEASED;        
      }      
  }

}

//--------------------------------------------------------------------|
// Sends a 30ms pulse to indicate half press                          |
//--------------------------------------------------------------------|
void halfpress(void)
{
  digitalWrite(cameraUsbPin, HIGH);
  delay(30);
  digitalWrite(cameraUsbPin, LOW); 
}

//--------------------------------------------------------------------|
// Sends a 150ms pulse to indicate full press                         |
//--------------------------------------------------------------------|
void fullpress(void)
{
  digitalWrite(cameraUsbPin, HIGH);
  delay(150);
  digitalWrite(cameraUsbPin, LOW); 
}

//--------------------------------------------------------------------|
// Sends two 30ms pulses to indicate button release                   |
//--------------------------------------------------------------------|
void release(void)
{
  digitalWrite(cameraUsbPin, HIGH);
  delay(30);
  digitalWrite(cameraUsbPin, LOW); 
  delay(30);
  digitalWrite(cameraUsbPin, HIGH); 
  delay(30);
  digitalWrite(cameraUsbPin, LOW);
}
